/******************************************************************************
 *
 * SYMPLiK CALLA
 * Copyright (C) 2010 Christopher Ho / SYMPLiK Tech. Co. Ltd.
 * All Rights Reserved, http://www.symplik.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * E-mail: christopher.ho@symplik.com
 *
 * ================================================
 *
 * $Archive: /TOOL/FLOWER/CALLA/src/symplik/flower/CONSOLE.java $
 * $Author: Christopher Ho $
 * $Date: 6/09/10 4:41a $
 * $Revision: 5 $
******************************************************************************/



package symplik.flower;

//~--- JDK imports ------------------------------------------------------------

import java.io.*;
import java.util.ArrayList;

/**
 * CONSOLE mode (text-based interactive) of CALLA.
 * 
 * @author  $Author: Christopher Ho $
 * @version $Revision: 5 $
 */
public class CONSOLE implements Constants {
    public static final String RCS_ID =
        "$Header: /TOOL/FLOWER/CALLA/src/symplik/flower/CONSOLE.java 5     6/09/10 4:41a Christopher Ho $";
    private String     answerStr = "";
    private Question   currentQuestion;
    private Response   responses = new Response();

    /**
     * Constructor
     */
    public CONSOLE() {
        System.setProperty(Helper.FLOWER_LOG_LEVEL, FlowerProp.instance().get(Helper.FLOWER_LOG_LEVEL));
        System.setProperty(Helper.FLOWER_LOG_OUTPUT, FlowerProp.instance().get(Helper.FLOWER_LOG_OUTPUT));
    }

    /**
     * Main class of Console - start this method directly if you want
     */
    public static void main(String[] args) {
      main();
    }
    
    protected static void main() {
      CONSOLE console = new CONSOLE();
      console.start();
    }

    private void start() {
        printTitle();

        String firstQuestion = Helper.getString(pStr("start"), CLASS_DEFAULT_START);
        Object clazz = Helper.getRenderObject(firstQuestion);
        if (clazz == null) {
          firstQuestion = CLASS_DEFAULT_START;
        }
        
        try {
            render(firstQuestion);
        } catch (Exception e) {
            showErrDialog(e.getClass().getName() + "\n" + e.getMessage());
            Helper.logError(e);
        }
    }

    private void response() {

        answerStr = "";
      
        try {
            if (currentQuestion.isPasswordInput()) {
                char password[] = PasswordField.getPassword(System.in, "Enter password ==");

                answerStr = String.valueOf(password);
            
            } else {
              
                System.out.print(">> ");

                InputStreamReader isr = new InputStreamReader(System.in);
                BufferedReader    br  = new BufferedReader(isr);

                answerStr = br.readLine();
            }
        } catch (IOException e) {
            showErrDialog(e.getMessage());
            Helper.logError(e);
        }

        if (currentQuestion.needAnswer()) {
          
            if (answerStr.trim().equals("")) {
                showErrDialog("Please enter a value.");
                response();
            }
        }

        try {
            if (currentQuestion.isMultipleChoices()) {
              
                if (answerStr.equalsIgnoreCase("X") && currentQuestion.showExit()) {
  
                  if (isRecordingMode()) {
                    responses.addResponse(currentQuestion,"X");
                  }                      
                  
                  exitProgram(0);
                }
  
                if (answerStr.equalsIgnoreCase("B") && currentQuestion.lastAction() != null) {
  
                  if (isRecordingMode()) {
                    responses.addResponse(currentQuestion,"B");
                  }
                  String clazz = currentQuestion.lastAction();
                  render(clazz);
                  return;
                }

                if (!Helper.isInteger(answerStr)) {
                    showErrDialog("Please enter a valid number.");
                    
                    response();
                }

                int idx = Integer.parseInt(answerStr);

                if ((idx > currentQuestion.choices().size()) || (idx <= 0)) {
                    showErrDialog("Please enter a valid number.");
                    
                    response();
                }

                // actual index is one less the entered number
                idx--;

                ArrayList<Choice> sortedChoices = Helper.sortChoices(
                                                    currentQuestion.choices(), 
                                                    currentQuestion.sortChoicesBy());  
                String answer = sortedChoices.get(idx).getChoiceKey();

                Answer.getInstance().putA(currentQuestion, answer);
                Helper.log(Helper.LOG_DEBUG, "Answer=" + answer);
            
            } else {

                // not a multiple choices...
                Answer.getInstance().putA(currentQuestion, answerStr);

                if (currentQuestion.isPasswordInput()) {
                    Helper.log(Helper.LOG_DEBUG, "Answer=******");
                } else {
                    Helper.log(Helper.LOG_DEBUG, "Answer=" + answerStr);
                }
            }

            boolean valid = currentQuestion.leaveQuestion();

            if (valid) {
              
              showMsgDialog(currentQuestion.leaveQuestionMsg(true));
              
              // save responses
                if (isRecordingMode()) {
                  responses.addResponse(currentQuestion, Answer.getInstance().getA(currentQuestion));
                }
              
                String clazz = currentQuestion.nextAction();

                Helper.log(LOG_DEBUG, "Next Action = " + clazz);

                render(clazz);

            } else {

                showMsgDialog(currentQuestion.leaveQuestionMsg(false));
                
                response();
            }

        } catch (Exception e) {
            showErrDialog(e.getClass().getName() + "\n" + e.getMessage());
            Helper.logError(e);
        }
    }

    private void render(String classAlias) {
      
        Object clazz = Helper.getRenderObject(classAlias);

        if (clazz == null) {
          exitProgram(1);
        }
        
        if (clazz instanceof Question) {
            Helper.log(Helper.LOG_DEBUG, ("render class is a Question"));
            currentQuestion = (Question) clazz;
            Helper.log(Helper.LOG_DEBUG, ("Class instantiation done."));

            if (currentQuestion.isFileInput()) {
              
              ConsoleFileBrowser cfg = new ConsoleFileBrowser();
              cfg.setExtension(currentQuestion.fileExtension());
              cfg.setShowExit(!currentQuestion.needAnswer());
              
              String file = cfg.selectFile();

              Answer.getInstance().putA(currentQuestion, file);
              
              boolean valid = currentQuestion.leaveQuestion();

              if (valid) {
                
                showMsgDialog(currentQuestion.leaveQuestionMsg(true));
                
                // save responses
                  if (isRecordingMode()) {
                    responses.addResponse(currentQuestion, Answer.getInstance().getA(currentQuestion));
                  }
                
                  String str = currentQuestion.nextAction();

                  Helper.log(LOG_DEBUG, "Next Action = " + clazz);
              
                  render(str);
              }
              
              
            } else if (currentQuestion.isDirectoryInput()) {

              ConsoleDirectoryBrowser cfg = new ConsoleDirectoryBrowser();
              cfg.setShowExit(!currentQuestion.needAnswer());
              
              String selectedDir = cfg.selectDirectory();

              Answer.getInstance().putA(currentQuestion, selectedDir);
              
              boolean valid = currentQuestion.leaveQuestion();
              
              if (valid) {
                
                showMsgDialog(currentQuestion.leaveQuestionMsg(true));
                
                // save responses
                  if (isRecordingMode()) {
                    responses.addResponse(currentQuestion, Answer.getInstance().getA(currentQuestion));
                  }
                
                  String str = currentQuestion.nextAction();

                  Helper.log(LOG_DEBUG, "Next Action = " + clazz);
              
                  render(str);
              }

            } else {
              Renderer renderer = new Renderer(currentQuestion);

              if (renderer.needToRender()) {
                StringBuffer buffer = renderer.getStringBuffer();

                System.out.println("\n" + buffer.toString());
                Helper.log(Helper.LOG_DEBUG, ("Class rendered."));
                
                response();
                
              } else {

                // no need to render
                Helper.log(Helper.LOG_DEBUG, ("Class skipped."));

                String str = currentQuestion.nextAction();

                render(str);
              }
           }
        }

        if (clazz instanceof Action) {
            Helper.log(Helper.LOG_DEBUG, ("render class is an Action"));

            Action action = (Action) clazz;

            // showSystemOut(action.showSystemOutput());
            Executor.execute(action);

            if (action.actionMessage() != null) {
                showMsgDialog(action.actionMessage());
            }

            String str = action.nextAction();

            render(str);
        }
    }

    private String pStr(String key) {
        return FlowerProp.instance().get("flower." + key);
    }

    private void showMsgDialog(Message msg) {
        if (msg != null) {
          System.out.println("[" + msg.getMessageTitle() + "] " + msg.getMessageContent());
        }
    }

    private void showErrDialog(String str) {
        showMsgDialog(new Message(Message.ERROR, str));
    }

    private void printTitle() {
        String       title = pStr("title");
        StringBuffer buf   = new StringBuffer();

        for (int i = 0; i < title.length() + 4; i++) {
            buf.append("-");
        }

        buf.append("\n  " + title + "  \n");

        for (int i = 0; i < title.length() + 4; i++) {
            buf.append("-");
        }
        
        if (isRecordingMode()) {
          buf.append("\n" + "(Recording Mode)");
        }
        
        System.out.println(buf.toString());
    }
    
    private boolean isRecordingMode() {
      return (Helper.getBoolean(FlowerProp.instance().get("flower.record"), false));
    }
    
    private void exitProgram(int exitCode) {
       if (isRecordingMode()) responses.writeResponseFile();
       
       System.exit(exitCode);
    }
}
