/******************************************************************************
 *
 * SYMPLiK CALLA
 * Copyright (C) 2010 Christopher Ho / SYMPLiK Tech. Co. Ltd.
 * All Rights Reserved, http://www.symplik.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * E-mail: christopher.ho@symplik.com
 *
 * ================================================
 *
 * $Archive: /TOOL/FLOWER/CALLA/src/symplik/flower/FlowerProp.java $
 * $Author: Christopher Ho $
 * $Date: 6/02/10 12:27a $
 * $Revision: 2 $
******************************************************************************/

package symplik.flower;

//~--- JDK imports ------------------------------------------------------------

import java.io.*;

import java.util.*;

/**
 * Singleton class to read flower.properties file
 * 
 * @author  $Author: Christopher Ho $
 * @version $Revision: 2 $
 */
public class FlowerProp implements Constants {
    public static final String RCS_ID =
        "$Header: /TOOL/FLOWER/CALLA/src/symplik/flower/FlowerProp.java 2     6/02/10 12:27a Christopher Ho $";
    
    private static FlowerProp flowerProp = null;
    private Properties        prop       = new Properties();

    /**
     * Constructor to load flower.properties file
     */
    protected FlowerProp() {
      
        String configFile = System.getProperty(FLOWER_CONFIG);

        File file;

        try {
          
          if (configFile==null || configFile.equals("")) {
              file = new File(System.getProperty("user.dir"), FLOWER_CONFIG_FILE);
              
              if (! file.exists()) {
                System.out.println("Unable to find " + file.getAbsolutePath());
                System.exit(1);
              }
              
          } else {
              file = new File(configFile);          
          }

          FileInputStream fis = new FileInputStream(file);

          prop.load(fis);
          
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * @return FlowerProp instance
     */
    public static FlowerProp instance() {
        if (flowerProp == null) {
            flowerProp = new FlowerProp();
        }

        return flowerProp;
    }

    /**
     * get properties value by key in flower.properties
     * @param key
     * @return properties value
     */
    public String get(String key) {
        return prop.getProperty(key);
    }
}
