/******************************************************************************
 *
 * SYMPLiK CALLA
 * Copyright (C) 2010 Christopher Ho / SYMPLiK Tech. Co. Ltd.
 * All Rights Reserved, http://www.symplik.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * E-mail: christopher.ho@symplik.com
 *
 * ================================================
 *
 * $Archive: /TOOL/FLOWER/CALLA/src/symplik/flower/GUI.java $
 * $Author: Christopher Ho $
 * $Date: 6/09/10 6:23a $
 * $Revision: 7 $
******************************************************************************/


package symplik.flower;

//~--- JDK imports ------------------------------------------------------------

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;

import java.io.*;
import java.util.ArrayList;

import javax.imageio.ImageIO;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.StyledDocument;

/**
 * Main class to start the GUI vesrion of CALLA
 * 
 * @author  $Author: Christopher Ho $
 * @version $Revision: 7 $
 */
public class GUI extends JFrame implements ActionListener, KeyListener, Constants {
    public static final String RCS_ID =
        "$Header: /TOOL/FLOWER/CALLA/src/symplik/flower/GUI.java 7     6/09/10 6:23a Christopher Ho $";
    private static final long serialVersionUID = 1764451497516196253L;
    private CallaJTextPane      textPane         = new CallaJTextPane();
    private StyledDocument      doc              = textPane.getStyledDocument();    

    private JButton             submitBTN        = new JButton("Submit " + '\u00BB');
    private JButton             selectBTN        = new JButton("Select...");
    private JButton             exitBTN          = new JButton("Exit");
    
    private JTextField          responseTF       = new JTextField();
    private JPasswordField      passwordTF       = new JPasswordField();
    
    private JLabel              responseLBL      = new JLabel();
    
    private JFileChooser        fc;
    private JFileChooser        dc;
    
    private JScrollPane         scrollPane;    
    private Question            currentQuestion;
    private Style               questionStyle;
    private Style               explanationStyle;
    private Style               choiceStyle;
    private static final String QUESTION_STYLE    = "question";
    private static final String EXPLANATION_STYLE = "explanation";
    private static final String CHOICE_STYLE      = "choice";
    
    private Response             responses        = new Response();  

    /**
     * Constructor
     */
    public GUI() {
        System.setProperty(Helper.FLOWER_LOG_LEVEL, FlowerProp.instance().get(Helper.FLOWER_LOG_LEVEL));
        System.setProperty(Helper.FLOWER_LOG_OUTPUT, FlowerProp.instance().get(Helper.FLOWER_LOG_OUTPUT));

        setProgramLookAndFeel();
        setStyles();
        
        GridBagConstraints c = new GridBagConstraints();
        
        // JScrollPane
        createTextPane();
        textPane.addKeyListener(this);
        scrollPane = new JScrollPane(textPane);

        scrollPane.setPreferredSize(new Dimension(Helper.getInt(pStr("panel.width"), 500),
                Helper.getInt(pStr("panel.height"), 400)));
        
        scrollPane.getViewport().setBackground(Helper.getColor(pStr("panel.background.color"), Color.WHITE));
        
        // responsePanel
        JPanel responsePanel = new JPanel(new GridBagLayout());

        createInputFields();
        c.gridx  = 0;
        c.gridy  = 0;
        c.anchor = GridBagConstraints.WEST;
        responsePanel.add(responseLBL, c);
        c.gridx  = 1;
        c.anchor = GridBagConstraints.WEST;
        responsePanel.add(responseTF, c);
        responsePanel.add(passwordTF, c);

        // buttonPanel
        JPanel buttonPanel = new JPanel(new GridBagLayout());

        c.gridy  = 0;
        c.gridx  = 0;
        c.anchor = GridBagConstraints.EAST;
        selectBTN.addActionListener(this);
        selectBTN.setEnabled(false);
        buttonPanel.add(selectBTN, c);
        c.gridx = 1;
        submitBTN.setMnemonic(KeyEvent.VK_ENTER);
        submitBTN.addActionListener(this);
        buttonPanel.add(submitBTN, c);
        c.gridx = 2;
        exitBTN.setMnemonic(KeyEvent.VK_X);
        exitBTN.addActionListener(this);
        buttonPanel.add(exitBTN, c);

        // overall layout
        this.setLayout(new GridBagLayout());
        c.gridy     = 0;
        c.gridx     = 0;
        c.gridwidth = 2;
        this.add(scrollPane, c);
        c.gridy     = 1;
        c.gridx     = 0;
        c.gridwidth = 1;
        this.add(responsePanel, c);
        c.gridy     = 1;
        c.gridx     = 1;
        c.gridwidth = 1;
        this.add(buttonPanel, c);
        String title = Helper.getString(FlowerProp.instance().get("flower.title"), "SYMPLiK CALLA");
        if (isRecordingMode()) {
          title = title + " (recording)";
        }
        this.setTitle(title);

        BufferedImage icon = null;

        try {
            icon = ImageIO.read(this.getClass().getResource(pStr("icon")));
            this.setIconImage(icon);            
        } catch (Exception e) {}

        this.pack();

        Dimension screenSize   = Toolkit.getDefaultToolkit().getScreenSize();
        int       windowWidth  = this.getWidth();
        int       windowHeight = this.getHeight();

        this.setLocation((int) (screenSize.getWidth() - windowWidth) / 2,
                         (int) (screenSize.getHeight() - windowHeight) / 2);
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        this.setResizable(false);
        this.setVisible(true);
    }

    /**
     * Main class of GUI - start this method directly if you want
     */
    public static void main(String[] args) {
      main();
    }
    
    protected static void main() {
      SwingUtilities.invokeLater(new Runnable() {
        public void run() {
            GUI gui = new GUI();
            gui.start();
        }
      });
    }
    
    private void start() {
        String firstQuestion = Helper.getString(FlowerProp.instance().get("flower.start"), CLASS_DEFAULT_START);
        Helper.log(LOG_DEBUG, "Start Question = " + firstQuestion);
        
        Object clazz = Helper.getRenderObject(firstQuestion);
        if (clazz == null) {
          firstQuestion = CLASS_DEFAULT_START;
        }
        
        try {
            render(firstQuestion);
        } catch (Exception e) {
            showErrDialog(e.getClass().getName() + "\n" + e.getMessage());
        }
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == exitBTN) {
            System.exit(0);
        }

        if (e.getSource() == submitBTN) {
            response();
        }

        if (e.getSource() == selectBTN) {
            if (currentQuestion.isDirectoryInput()) {
                showDirectoryDialog();
            }

            if (currentQuestion.isFileInput()) {
                showFileDialog();
            }
        }
    }

    public void keyPressed(KeyEvent e) {
        if (e.getKeyChar() == KeyEvent.VK_ENTER) {
            e.consume();
            response();
        }
    }

    private void response() {
        String ans = null;

        if (currentQuestion.isPasswordInput()) {
            ans = new String(passwordTF.getPassword());
            passwordTF.setText("");
        } else {
            ans = responseTF.getText().trim();
            responseTF.setText("");
        }

        if (currentQuestion.needAnswer()) {
            if (ans.equals("")) {
                showErrDialog("Please enter a value.");

                return;
            }
        }

        try {
            if (currentQuestion.isMultipleChoices()) {
                if (ans.equalsIgnoreCase("X") && currentQuestion.showExit()) {

                  if (isRecordingMode()) {
                    responses.addResponse(currentQuestion,"X");
                  }                      
                  
                  exitProgram(0);
                }

                if (ans.equalsIgnoreCase("B") && currentQuestion.lastAction() != null) {

                  if (isRecordingMode()) {
                    responses.addResponse(currentQuestion,"B");
                  }
                  String clazz = currentQuestion.lastAction();
                  inProgress(true);
                  render(clazz);
                  inProgress(false);
                  return;
                }
                
                if (!Helper.isInteger(ans)) {
                    showErrDialog("Please enter a valid number.");

                    return;
                }

                int idx = Helper.getInt(ans, -1);

                if (idx > currentQuestion.choices().size() || idx <= 0) {
                    showErrDialog("Please enter a valid number.");

                    return;
                }

                // actual index is one less the entered number
                idx--;

                ArrayList<Choice> sortedChoices = Helper.sortChoices(
                                                    currentQuestion.choices(), 
                                                    currentQuestion.sortChoicesBy());  
                String answer = sortedChoices.get(idx).getChoiceKey();

                Answer.getInstance().putA(currentQuestion, answer);
                Helper.log(Helper.LOG_DEBUG, "Answer=" + answer);
            } else {

                // not a multiple choices...
                Answer.getInstance().putA(currentQuestion, ans.trim());

                if (currentQuestion.isPasswordInput()) {
                    Helper.log(Helper.LOG_DEBUG, "Answer=******");
                } else {
                    Helper.log(Helper.LOG_DEBUG, "Answer=" + ans);
                }
            }

            boolean valid = currentQuestion.leaveQuestion();

            if (valid) {
              
              showMsgDialog(currentQuestion.leaveQuestionMsg(true));
              
              // save responses
                if (isRecordingMode()) {
                    responses.addResponse(currentQuestion,
                                          Answer.getInstance().getA(currentQuestion));
                }              
              
                String clazz = currentQuestion.nextAction();

                inProgress(true);
                render(clazz);
                inProgress(false);
            } else {

                showMsgDialog(currentQuestion.leaveQuestionMsg(false));
                
            }
        } catch (Exception e) {
            showErrDialog(e.getClass().getName() + "\n" + e.getMessage());
            Helper.logError(e);
        }
    }

    private void render(String classAlias) {
        
        Object clazz = Helper.getRenderObject(classAlias);
        if (clazz == null) {
          exitProgram(1);
        }

        if (clazz instanceof Question) {
            Helper.log(Helper.LOG_DEBUG, ("render class is a Question"));
            currentQuestion = (Question) clazz;
            Helper.log(Helper.LOG_DEBUG, ("Class instantiation done."));

            Renderer renderer = new Renderer(currentQuestion);

            if (renderer.needToRender()) {
                showMsgDialog(currentQuestion.enterQuestionMsg(true));  
                
                textPane.setLineWrap(currentQuestion.lineWrap());
                selectBTN.setEnabled(false);
                responseTF.setEditable(true);
                exitBTN.setEnabled(currentQuestion.showExit());   
                
                String title = currentQuestion.getTitle();
                if (title != null) {
                  if (Helper.getBoolean(FlowerProp.instance().get("flower.record"), false)) {
                    title = title + " (Recording Mode)";
                  }                  
                  this.setTitle(title);
                }
                
                StringBuffer questionBuf     = renderer.getQuestion();
                StringBuffer explanationBuf  = renderer.getExplanation();
                StringBuffer choiceBuf       = renderer.getChoices();

                if (currentQuestion.isNewScreen()) textPane.setText(null);

                addText(questionBuf.toString(), QUESTION_STYLE);
                addText(explanationBuf.toString(), EXPLANATION_STYLE);
                addText("\n", EXPLANATION_STYLE);
                addText(choiceBuf.toString(), CHOICE_STYLE);

                if (currentQuestion.isPasswordInput()) {
                    passwordTF.setVisible(true);
                    responseTF.setVisible(false);
                    passwordTF.requestFocus();
                } else {
                    passwordTF.setVisible(false);
                    responseTF.setVisible(true);
                    responseTF.requestFocus();
                }

                if (currentQuestion.isDirectoryInput()) {
                    showDirectoryDialog();
                }

                if (currentQuestion.isFileInput()) {
                    showFileDialog();
                }

                Helper.log(Helper.LOG_DEBUG, ("Class rendered."));
                
            } else {
                
                showMsgDialog(currentQuestion.enterQuestionMsg(false));  
                // no need to render
                Helper.log(Helper.LOG_DEBUG, ("Class skipped."));

                String str = currentQuestion.nextAction();

                render(str);
            }
        }

        if (clazz instanceof Action) {
            Helper.log(Helper.LOG_DEBUG, ("render class is an Action"));

            Action action = (Action) clazz;

            showSystemOut(action.showSystemOutput());
            Executor.execute(action);

            if (action.actionMessage() != null) {
                showMsgDialog(action.actionMessage());
            }

            String str = action.nextAction();

            render(str);
        }
        
        // put the cursor on the first line
        textPane.setCaretPosition(0);

    }

    public void keyReleased(KeyEvent e) {

        // Auto-generated method stub
    }

    public void keyTyped(KeyEvent e) {
      if (e.getSource()==textPane) {
        if (currentQuestion.isPasswordInput()) this.passwordTF.requestFocus();
        if (currentQuestion.isMultipleChoices() ||
            currentQuestion.isFileInput() ||
            currentQuestion.isDirectoryInput()) this.responseTF.requestFocus();
            char c = e.getKeyChar();
            if (c >=32  && c <= 126) {
              this.responseTF.setText(this.responseTF.getText() + c);
            }
      }
    }

    private void setProgramLookAndFeel() {
        try {
            String lnfClass = pStr("lnf");

            UIManager.setLookAndFeel(lnfClass);
        } catch (Exception e) {}
    }

    private String pStr(String key) {
        return FlowerProp.instance().get("flower.gui." + key);
    }

    private void showMsgDialog(Message msg) {
      if (msg != null) {
        JOptionPane.showMessageDialog(this, msg.getMessageContent(), msg.getMessageTitle(), msg.getMessageType());
      }
    }

    private void showErrDialog(String str) {
        showMsgDialog(new Message(Message.ERROR, str));
    }

    private void showSystemOut(boolean flag) {
        if (flag) {
            OutputStream out = new OutputStream() {
                public void write(int b) throws IOException {
                    addText(String.valueOf((char) b), CHOICE_STYLE);
                }
                public void write(byte[] b, int off, int len) throws IOException {
                    addText(new String(b, off, len), CHOICE_STYLE);
                }
                public void write(byte[] b) throws IOException {
                    write(b, 0, b.length);
                }
            };

            System.setOut(new PrintStream(out, true));
            System.setErr(new PrintStream(out, true));
        } else {
            System.setOut(System.out);
            System.setErr(System.err);
        }
    }

    private void addText(String str, String style) {
              try {
                doc.insertString(doc.getLength(), str, doc.getStyle(style));
                
              } catch (BadLocationException ble) {
                Helper.logError(ble);
              }
    }

    private void inProgress(boolean flag) {
        this.submitBTN.setEnabled(!flag);
        this.responseTF.setEnabled(!flag);
        this.passwordTF.setEnabled(!flag);
        this.textPane.setEnabled(!flag);

        if (flag) {
            this.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        } else {
            this.setCursor(Cursor.getDefaultCursor());
            this.responseTF.requestFocus();
        }
    }

    private void createTextPane() {
        textPane.setEditable(false);
        textPane.setFocusable(true);
        textPane.setBackground(Helper.getColor(pStr("panel.background.color"), Color.WHITE));
    }

    private void createInputFields() {
        Font rfont = new Font(Helper.getString(pStr("response.font.name"), "Serif"),
                              Helper.getFontStyle(pStr("response.font.style"), Font.BOLD),
                              Helper.getInt(pStr("response.font.size"), 16));
        Color bg = Helper.getColor(pStr("response.background.color"), Color.WHITE);
        Color fg = Helper.getColor(pStr("response.font.color"), Color.BLACK);
        Color cc = Helper.getColor(pStr("response.caret.color"), Color.BLACK);

        responseLBL.setText(" => ");
        responseLBL.setFont(rfont);
        responseLBL.setBackground(bg);
        responseLBL.setForeground(fg);
        responseTF.setColumns(Helper.getInt(pStr("response.size"), 30));
        responseTF.setFont(rfont);
        responseTF.setBorder(BorderFactory.createEmptyBorder());
        responseTF.setBackground(bg);
        responseTF.setForeground(fg);
        responseTF.setCaretColor(cc);
        responseTF.addKeyListener(this);

        // overlay both textfields; set the passwordTF hidden first
        passwordTF.setColumns(Helper.getInt(pStr("response.size"), 30));
        passwordTF.setFont(rfont);
        passwordTF.setBorder(BorderFactory.createEmptyBorder());
        passwordTF.setBackground(bg);
        passwordTF.setForeground(fg);
        passwordTF.setCaretColor(cc);
        passwordTF.addKeyListener(this);
    }

    private void showDirectoryDialog() {
        selectBTN.setEnabled(true);
        responseTF.setEditable(false);

        String currentDir = Helper.getString(
            Answer.getInstance().getB(FILE_BROWSER_CURR_DIR), 
            System.getProperty("user.dir"));
        
        dc = new JFileChooser(currentDir);
        
        dc.setCurrentDirectory(new File(currentDir));
        dc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        
        int returnVal = dc.showOpenDialog(this);

        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = dc.getSelectedFile();

            responseTF.setText(file.getAbsolutePath());
            Answer.getInstance().putB(FILE_BROWSER_CURR_DIR, file.getAbsolutePath());
        }
    }

    private void showFileDialog() {
        selectBTN.setEnabled(true);
        responseTF.setEditable(false);
        
        String currentDir = Helper.getString(
            Answer.getInstance().getB(FILE_BROWSER_FILE), 
            System.getProperty("user.dir"));
        
        fc = new JFileChooser(currentDir);
        
        if (currentQuestion.fileExtension()!=null) {
          fc.addChoosableFileFilter(new FileFilter() {
              String filter = currentQuestion.fileExtension();
              public boolean accept(File f) {
                  if (f.isDirectory()) {
                      return true;
                  }
  
                  String extension = Helper.getExtension(f);
  
                  if ((extension != null) &&!extension.equals("")) {
                      if (extension.equalsIgnoreCase(filter)) {
                          return true;
                      } else {
                          return false;
                      }
                  } else {
                      return true;
                  }
              }
              public String getDescription() {
                  return filter.toUpperCase() + " file only";
              }
          });
        }

        int returnVal = fc.showOpenDialog(this);

        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();

            responseTF.setText(file.getAbsolutePath());
            Answer.getInstance().putB(FILE_BROWSER_FILE, file.getAbsolutePath());
        }
    }
    
    private void setStyles() {
      Style defStyle = StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE);

      questionStyle = doc.addStyle(QUESTION_STYLE, defStyle);
      StyleConstants.setBackground(questionStyle, Helper.getColor(pStr("question.background.color"), Color.WHITE));      
      StyleConstants.setForeground(questionStyle, Helper.getColor(pStr("question.font.color"), Color.BLACK));
      StyleConstants.setFontFamily(questionStyle, pStr("question.font.name"));
      StyleConstants.setFontSize(questionStyle, Helper.getInt(pStr("question.font.size"), 14));
      if (pStr("question.font.style").toUpperCase().equals("ITALIC")) {
        StyleConstants.setItalic(questionStyle, true);
      }
      if (pStr("question.font.style").toUpperCase().equals("BOLD")) {
        StyleConstants.setBold(questionStyle, true);
      }
      
      choiceStyle = doc.addStyle(CHOICE_STYLE, defStyle);
      StyleConstants.setBackground(choiceStyle, Helper.getColor(pStr("choice.background.color"), Color.WHITE));      
      StyleConstants.setForeground(choiceStyle, Helper.getColor(pStr("choice.font.color"), Color.BLACK));
      StyleConstants.setFontFamily(choiceStyle, pStr("choice.font.name"));
      StyleConstants.setFontSize(choiceStyle, Helper.getInt(pStr("choice.font.size"), 14));
      if (pStr("choice.font.style").toUpperCase().equals("ITALIC")) {
        StyleConstants.setItalic(choiceStyle, true);
      }
      if (pStr("choice.font.style").toUpperCase().equals("BOLD")) {
        StyleConstants.setBold(choiceStyle, true);
      }

      explanationStyle = doc.addStyle(EXPLANATION_STYLE, defStyle);
      StyleConstants.setBackground(explanationStyle, Helper.getColor(pStr("explanation.background.color"), Color.WHITE));      
      StyleConstants.setForeground(explanationStyle, Helper.getColor(pStr("explanation.font.color"), Color.BLACK));
      StyleConstants.setFontFamily(explanationStyle, pStr("explanation.font.name"));
      StyleConstants.setFontSize(explanationStyle, Helper.getInt(pStr("explanation.font.size"), 14));
      if (pStr("explanation.font.style").toUpperCase().equals("ITALIC")) {
        StyleConstants.setItalic(explanationStyle, true);
      }
      if (pStr("explanation.font.style").toUpperCase().equals("BOLD")) {
        StyleConstants.setBold(explanationStyle, true);
      }      
      
    }
    
    private boolean isRecordingMode() {
       return (Helper.getBoolean(FlowerProp.instance().get("flower.record"), false));
    }
    
    private void exitProgram(int exitCode) {
      if (isRecordingMode()) responses.writeResponseFile();
      
      System.exit(exitCode);
   }    
}
