/******************************************************************************
 *
 * SYMPLiK CALLA
 * Copyright (C) 2010 Christopher Ho / SYMPLiK Tech. Co. Ltd.
 * All Rights Reserved, http://www.symplik.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * E-mail: christopher.ho@symplik.com
 *
 * ================================================
 *
 * $Archive: /TOOL/FLOWER/CALLA/src/symplik/flower/Renderer.java $
 * $Author: Christopher Ho $
 * $Date: 6/09/10 6:23a $
 * $Revision: 5 $
******************************************************************************/



package symplik.flower;

//~--- JDK imports ------------------------------------------------------------

import java.util.ArrayList;

/**
 * Renderer is responsible for creating textual elements (as StringBuffer) for Question class.
 * Elements are: question, explanation, and choices
 * 
 * @author  $Author: Christopher Ho $
 * @version $Revision: 5 $
 */
public class Renderer implements Constants {
    public static final String RCS_ID =
        "$Header: /TOOL/FLOWER/CALLA/src/symplik/flower/Renderer.java 5     6/09/10 6:23a Christopher Ho $";
    private StringBuffer questionBuf    = new StringBuffer();
    private StringBuffer choiceBuf      = new StringBuffer();
    private StringBuffer explanationBuf = new StringBuffer();
    private static final String sep     = System.getProperty("line.separator");
    private boolean      needRender;

    /**
     * Constructor
     * @param class subclassed of Question
     */
    public Renderer(Question question) {
        Helper.log(Helper.LOG_DEBUG, ("Rendering start..."));
        questionBuf.setLength(0);
        explanationBuf.setLength(0);        
        choiceBuf.setLength(0);

        boolean valid = question.enterQuestion();

        Helper.log(Helper.LOG_DEBUG, ("enterQuestion=" + valid));

        if (valid) {
            needRender = true;

            // display question
            addLine(question.getQuestion(), questionBuf);

            addLine(question.getExplanation(), explanationBuf);
            
            // display choices
            if (question.isMultipleChoices()) {
                ArrayList<Choice> choices = question.choices();

                if (choices != null) {
                  choices = Helper.sortChoices(choices, question.sortChoicesBy());
                  displayChoices(choices);
                }
                
                if (question.lastAction() != null) {
                  add("[B]  Back  ", choiceBuf);
                }
               
                if (question.showExit()) {
                  add("[X]  Exit", choiceBuf);
                }
            }
        } else {
            needRender = false;
        }
    }

    /**
     * Question.enterQuestion value
     * @return True or False
     */
    public boolean needToRender() {
        return needRender;
    }

    /**
     * @return String content of Question as StringBuffer
     */
    public StringBuffer getQuestion() {
      return questionBuf;
    }

    /**
     * @return String content of Explanation as StringBuffer
     */
    public StringBuffer getExplanation() {
      return explanationBuf;
    }

    /**
     * @return String content of multiple choices as StringBuffer
     */
    public StringBuffer getChoices() {
      return choiceBuf;
    }

    /**
     * @return combination of Question, Explanation and Choices
     */
    public StringBuffer getStringBuffer() {
        return questionBuf.append(explanationBuf).append(sep).append(choiceBuf);
    }

    private void addLine(StringBuffer buffer) {
        buffer.append(sep);
    }

    private void add(String str, StringBuffer buffer) {
      if ((str != null) &&!str.trim().equals("")) {
        buffer.append(str);
      }
    }
    
    private void addLine(String str, StringBuffer buffer) {
        if ((str != null) &&!str.trim().equals("")) {
            buffer.append(str + sep);
        }
    }
    
    private void displayChoices(ArrayList<Choice> choices) {
      // the first choice is 1 (i is starting from zero)
      
      for (int i = 0; i < choices.size(); i++) {
        Choice c = choices.get(i);
        if (i < 9) {
          addLine("[" + (i + 1) + "]  " + c.getChoiceDesc(), choiceBuf);
        } else {
          addLine("[" + (i + 1) + "] " + c.getChoiceDesc(), choiceBuf);
        }
      }
      
      addLine(choiceBuf);
   }
}
