/******************************************************************************
 *
 * SYMPLiK CALLA
 * Copyright (C) 2010 Christopher Ho / SYMPLiK Tech. Co. Ltd.
 * All Rights Reserved, http://www.symplik.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * E-mail: christopher.ho@symplik.com
 *
 * ================================================
 *
 * $Archive: /TOOL/FLOWER/CALLA/src/symplik/flower/Response.java $
 * $Author: Christopher Ho $
 * $Date: 6/02/10 12:27a $
 * $Revision: 2 $
******************************************************************************/

package symplik.flower;

import java.io.*;
import java.text.SimpleDateFormat;
import java.util.*;

import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.jdom.xpath.XPath;

/**
 * Response class is responsible for reading and creating response.xml for silent operation.
 * The structure of response.xml is:
 * <pre>
 * {@code
 * <?xml version="1.0" encoding="UTF-8"?>
 * <RESPONSES CREATION_DATE="[timestamp]">
 *   <RESPONSE ID="[sequence no]" CLASS="[Question full-qualified class name]">[ChoiceKey | free-text answer]</RESPONSE>
 *   <RESPONSE ID="0" CLASS="symplik.oracle.fndload.question.OperationModeQuestion">DOWNLOAD</RESPONSE>
 *   <RESPONSE ID="1" CLASS="symplik.oracle.fndload.question.DownloadObjectQuestion">FND_NEW_MESSAGES</RESPONSE>
 * </RESPONSES>
 * }
 * </pre>  
 *   
 * @author  $Author: Christopher Ho $
 * @version $Revision: 2 $
 */
public class Response implements Constants {
    public static final String RCS_ID =
      "$Header: /TOOL/FLOWER/CALLA/src/symplik/flower/Response.java 2     6/02/10 12:27a Christopher Ho $";
  
    private Vector<String> questionV = new Vector<String>();
    private Vector<String> answerV   = new Vector<String>();
    
    public Response() {
    }
    
    /**
     * add a response node to the response file
     * @param clazz
     * @param value
     */
    public void addResponse(Question clazz, String value) {
      questionV.add(clazz.getClass().getCanonicalName());
      answerV.add(value);
    }
    
    /**
     * write the response xml file
     */
    public void writeResponseFile() {
      String str = Helper.getString(FlowerProp.instance().get(FLOWER_RESPONSES), FLOWER_RESPONSES_FILE);
      File file = new File(str);
      Helper.log(LOG_DEBUG, "write response file " + file.getAbsolutePath());
      
      Element          root       = new Element("RESPONSES");
      SimpleDateFormat sdf        = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
      String           timestamp  = sdf.format(Calendar.getInstance().getTime());
      
      root.setAttribute("CREATION_DATE", timestamp);
      
      for (int i=0;i<questionV.size();i++) {
        Element ele = new Element("RESPONSE");
        ele.setAttribute("ID", String.valueOf(i));
        ele.setAttribute("CLASS", questionV.get(i));
        ele.setText(answerV.get(i));
        root.addContent(ele);
      }

      Document document = new Document(root);
     
      try {
        
        XMLOutputter outputter = new XMLOutputter();

        outputter.setFormat(Format.getPrettyFormat());
        outputter.output(document, new FileOutputStream(file));
        
      } catch (IOException ioe) {
        Helper.logError(ioe);
      }
    }
    
    public String getClassName(int i) {
      return questionV.get(i);
    }
    
    public String getAnswer(int i) {
      return answerV.get(i);
    }
    
    /**
     * 
     */
    public void readResponseFile() {
      String f = Helper.getString(FlowerProp.instance().get(FLOWER_RESPONSES), FLOWER_RESPONSES_FILE);
      File file = new File(f);
      if (!file.exists()) {
        System.out.println("Unable to find responses file " + file.getAbsolutePath());
        System.exit(1);
      } else {
        Helper.log(Helper.LOG_DEBUG, "Using response file " + file.getAbsolutePath());
        
        try {
          SAXBuilder builder    = new SAXBuilder();          
          Document   responses  = builder.build(file);
          
          XPath xpath = XPath.newInstance("/RESPONSES/RESPONSE");
          List<Element>  list  = xpath.selectNodes(responses);
          
          String[] questionArry = new String[list.size()];
          String[] answerArry   = new String[list.size()];

          Element   ele;
          Attribute attr;
          
          for (int i=0;i<list.size();i++) {
            xpath = XPath.newInstance("/RESPONSES/RESPONSE[@ID=" + i + "]/@CLASS");
            attr   = (Attribute)xpath.selectSingleNode(responses);
            questionArry[i] = attr.getValue();
            
            xpath = XPath.newInstance("/RESPONSES/RESPONSE[@ID=" + i + "]");
            ele   = (Element)xpath.selectSingleNode(responses);
            answerArry[i] = ele.getText();
          }
          
          questionV.addAll(Arrays.asList(questionArry));
          answerV.addAll(Arrays.asList(answerArry));
          
        } catch (JDOMException je) {
          Helper.log(LOG_ERROR, "Malformat responses XML file");
          System.exit(1);
        } catch (IOException ioe) {
          Helper.log(LOG_ERROR, "Unable to read responses XML file");          
          System.exit(1);          
        }
      }
    }
}


